import fs from 'fs';
import { PDFDocument, PDFImage, PDFPage } from 'pdf-lib';
import { Request, Response } from 'express';
import { getNtnuiProfile } from 'ntnui-tools';
import { base64ToPdf, getPdfAsBase64 } from '../utils/base64Util';

const pdfPath = 'src/assets/template.pdf';

async function editPdfForSender(req: Request, res: Response) {
	try {
		let pdfDoc;
		let isFirstEdit = Boolean(req.body.contract === '');

		//If the contract is being edited for the first time names, the contract is being
		//generated from the template
		if (isFirstEdit) {
			pdfDoc = await PDFDocument.load(fs.readFileSync(pdfPath));
		} else {
			//Second edit gets the previously edited contract as a bas64 string and inserts
			//the signatures
			pdfDoc = await base64ToPdf(req.body.contract);
		}

		let page = pdfDoc.getPage(0);

		//Inserts names, groups, roles and signatures
		await editContract(pdfDoc, page, isFirstEdit, req, res);

		//The edited contract is returned as a base64 string
		let base64pdf;
		await getPdfAsBase64(pdfDoc)
			.then((data) => {
				base64pdf = data;
			})
			.catch((error) => {
				console.log(error);
				res.status(500).send({ message: 'Error encoding the PDF' });
			});
		return res.status(200).json({ message: 'PDF edited', pdf: base64pdf });
	} catch (error) {
		console.error(error);
		return res.status(500).send({ message: 'Error editing PDF' });
	}
}

//Gets a base64 encoded signature and inserts it into the base64 encoded contract
async function editPdfForReceiver(req: Request, res: Response) {
	try {
		let pdfDoc = await base64ToPdf(req.body.contract);
		let page = pdfDoc.getPage(0);

		//The split is to isolate the base64 string without the file-information
		const base64png = req.body.signature.split(',')[1];
		let signatureImage = await pdfDoc.embedPng(
			Buffer.from(base64png, 'base64')
		);
		insertSignature(page, signatureImage, 70, 150);

		//The edited contract is returned as a base64 string
		let base64pdf;
		await getPdfAsBase64(pdfDoc)
			.then((data) => {
				base64pdf = data;
			})
			.catch((error) => {
				console.log(error);
				res.status(500).send({ message: 'Error encoding the PDF' });
			});
		return res.status(200).json({ message: 'PDF edited', pdf: base64pdf });
	} catch (error) {
		console.error(error);
		return res.status(500).send({ message: 'Error editing PDF' });
	}
}

async function editContract(
	pdfDoc: PDFDocument,
	page: PDFPage,
	isFirstEdit: boolean,
	req: Request,
	res: Response
) {
	if (isFirstEdit) {
		//First request before signatures are being sent
		let userInfo: ContractUserInfo[] | null = [];
		await getUserInfo(req)
			.then((data) => {
				userInfo = data;
			})
			.catch((error) => {
				console.log(error);
				return res.status(500).send({
					message: 'Could not fetch user info'
				});
			});
		//Inserts names, groups and roles
		insertUserInfo(page, userInfo[0], userInfo[1]);
		if (req.body.signature) {
			const base64png = req.body.signature.split(',')[1];
			let signatureImage = await pdfDoc.embedPng(
				Buffer.from(base64png, 'base64')
			);
			insertSignature(page, signatureImage, 350, 150);
		}
	} else {
		const base64png = req.body.signature.split(',')[1];
		let signatureImage = await pdfDoc.embedPng(
			Buffer.from(base64png, 'base64')
		);
		insertSignature(page, signatureImage, 350, 150);
	}
}

async function getUserInfo(req: Request) {
	let { accessToken } = req.cookies;
	const loggedInUser = (await getNtnuiProfile(accessToken)).data;

	let senderGroup = req.body.group.name;
	let senderMembershipGroups = loggedInUser.memberships.map(
		(membership) => membership.group
	);
	if (senderMembershipGroups.includes('Hovedstyret'))
		senderGroup = 'Hovedstyret';

	let senderMembership = loggedInUser.memberships.filter(
		(membership) => membership.group === senderGroup
	)[0];

	let senderRole

	if (senderMembership) {
		senderRole = senderMembership.type;
	} else {
		senderRole = "Tillitsvalgt"
	}

	let senderInfo: ContractUserInfo = {
		first_name: loggedInUser.first_name,
		last_name: loggedInUser.last_name,
		type: senderRole,
		group: senderGroup
	};

	let receiverInfo: ContractUserInfo = {
		first_name: req.body.first_name,
		last_name: req.body.last_name,
		type: req.body.type,
		group: req.body.group.name
	};

	return [senderInfo, receiverInfo];
}

type ContractUserInfo = {
	first_name: string;
	last_name: string;
	type: string;
	group: string;
};

function insertUserInfo(
	page: PDFPage,
	senderInfo: ContractUserInfo,
	receiverInfo: ContractUserInfo
) {
	//The coordinates must be changed if the position of the 
	//sign-fields on the contract is moved
	page.drawText(
		`${senderInfo.first_name} ${senderInfo.last_name}\n` +
			`${translateRole(senderInfo.type)}, NTNUI ${senderInfo.group}`,
		{
			x: 360,
			y: 130,
			size: 11
		}
	);
	page.drawText(
		`${receiverInfo.first_name} ${receiverInfo.last_name}\n` +
			`${translateRole(receiverInfo.type)}, NTNUI ${receiverInfo.group}`,
		{
			x: 70,
			y: 130,
			size: 11
		}
	);
}

function insertSignature(page: PDFPage, image: PDFImage, x: number, y: number) {
	page.drawImage(image, {
		x: x,
		y: y,
		width: 200,
		height: 60
	});
}

const translateRole = (role: string) => {
	switch (role) {
		case 'leader':
			return 'Leder';
		case 'deputy_leader':
			return 'Nestleder';
		case 'cashier':
			return 'Kasserer';
		case 'board_member':
			return 'Styremedlem';
		case 'deputy_board_member':
			return 'Varamedlem';
		case 'volunteer':
			return 'Frivillig';
		default:
			return 'Tillitsvalgt';
	}
};

export { editPdfForSender, editPdfForReceiver };