import { Request, Response } from 'express';
import { getDb } from '../db';
import { Group } from '../types/group';
import { getNtnuiProfile } from 'ntnui-tools';
import axios from 'axios';

const addGroup = async (groups: Group[]) => {
	const dbConnection = getDb();

	dbConnection
		.collection('groups')
		.insertMany(groups)
		.catch((error: any) => {
			console.log(error);
			throw new Error('Could not post group');
		});
};

const getGroups = async (req: Request, res: Response) => {
	const dbConnection = getDb();

	let { accessToken } = req.cookies;
	const user = (await getNtnuiProfile(accessToken)).data;

	let groups: Group[] = [];
	let membershipsystemGroups: Group[] = [];
	let ndaGroups: Group[] = [];

	//Fetches all groups from the membershipsystem
	await axios
		.get(`https://dev.api.ntnui.no/groups`)
		.then((response) => {
			membershipsystemGroups = response.data.results;
		})
		.catch((error) => {
			console.log(error);
			return res
				.status(500)
				.json({ error: 'Could not fetch users from membershipsystem' });
		});

	//Fetches all groups from the nda system
	await dbConnection
		.collection('groups')
		.find({}, { id: 0 })
		.forEach((group: Group) => ndaGroups.push(group))
		.catch(() => {
			return res.status(500).json({ error: 'Could not fetch groups' });
		});

	let ndaGroupSlugs = ndaGroups.map((group: Group) => group.slug);
	let nonDuplicateMembershipGroups = membershipsystemGroups.filter(
		(group: Group) => !ndaGroupSlugs.includes(group.slug)
	);
	groups.push(...nonDuplicateMembershipGroups);
	groups.push(...ndaGroups);

	//Post groups that exist in the membershipsystem but not in the nda system
	if (nonDuplicateMembershipGroups.length > 0) {
		try {
			await addGroup(nonDuplicateMembershipGroups);
		} catch (error) {
			console.log(error);
			return res.status(500).send({ error: error });
		}
	}

	//Members of Hovedstyret get to see all groups
	const userInHs = user.memberships.some(
		(membership: { slug: string }) => membership.slug == 'hovedstyret'
	);
	if (userInHs)
		return res
			.status(200)
			.json({ message: 'Successful fetch', groups: groups });

	//Users who are not in Hovedstyret only get to see groups they are leaders of
	const groupslugsInMemberships = user.memberships.map((membership) => {
		return { slug: membership.slug, type: membership.type };
	});
	const groupsInMemberships = groups.filter((group: Group) =>
		groupslugsInMemberships.some(
			(membership) =>
				membership.slug === group.slug && membership.type === 'leader'
		)
	);
	return res
		.status(200)
		.json({ message: 'Successful fetch', groups: groupsInMemberships });
};

export { addGroup, getGroups };