import { Request, Response } from 'express';
import { getDb } from '../db';
import { v4 as uuidv4 } from 'uuid';
import { sendMailForSigning } from './mailService';
import { getNtnuiProfile } from 'ntnui-tools';
import { User } from '../types/user';
import axios from 'axios';

const getUsers = async (req: Request, res: Response) => {
	const dbConnection = getDb();

	let { accessToken } = req.cookies;
	const user = (await getNtnuiProfile(accessToken)).data;

	let users: User[] = [];
	let membershipsystemUsers: User[] = [];
	let ndaUsers: User[] = [];

	let ignoreInactiveUsers = false;
	//Gets all board-members of the given group from the membershipsystem
	await axios
		.get(
			`https://dev.api.ntnui.no/groups/${req.query.groupslug}/board-members/`
		)
		.then((response) => {
			membershipsystemUsers = response.data;
		})
		.catch((error) => {
			//Error code 404 represent "Not Found", which in this context,
			//implies that the group does not exist in the memebershipsystem anymore
			//This is done to ignore the membershipsystem call if and only if the group
			//does not exist anymore, but still throw errors if the api is down or such
			if (error.response.status === 404) {
				ignoreInactiveUsers = true;
			} else {
				return res
					.status(500)
					.json({
						error: 'Could not fetch users from membershipsystem'
					});
			}
		});

	const projection = {
		'contract.contract_pdf': 0,
		signature: 0,
		memberships: 0
	};
	//If the user is in HS he should be able to see all board-members for a given group who has 
	//signed an NDA, not just the current ones
	if (
		user.memberships.some(
			(membership: { slug: string }) => membership.slug == 'hovedstyret'
		)
	) {
		await dbConnection
			.collection('users')
			.find({ 'contract.group_slug': req.query.groupslug }, projection)
			.forEach((user: User) => ndaUsers.push(user))
			.catch(() => {
				return res
					.status(500)
					.json({ error: 'Could not fetch nda users' });
			});
	} else {
		//Group leaders only get to see the currently active board-members in their group
		let activeEmails = membershipsystemUsers.map(
			(user: User) => user.email
		);

		await dbConnection
			.collection('users')
			.find(
				{
					'contract.group_slug': req.query.groupslug,
					email: { $in: activeEmails }
				},
				projection
			)
			.forEach((user: User) => ndaUsers.push(user))
			.catch(() => {
				return res
					.status(500)
					.json({ error: 'Could not fetch nda users' });
			});
	}

	if (!ignoreInactiveUsers) {
		let ndaDatabaseEmails = ndaUsers.map((user: User) => user.email);
		let nonDuplicateMembershipUsers = membershipsystemUsers.filter(
			(user: User) => !ndaDatabaseEmails.includes(user.email)
		);
		users.push(...nonDuplicateMembershipUsers);
	}
	users.push(...ndaUsers);
	
	return res.status(200).json({ message: 'Successful fetch', users: users });
};

const getLoggedInUser = async (req: Request, res: Response) => {

	let { accessToken } = req.cookies;
	const user = await getNtnuiProfile(accessToken);
	return res
		.status(200)
		.json({ message: 'Successful fetch', user: user.data });
};


const addUser = async (req: Request, res: Response) => {
	const dbConnection = getDb();
	const user = req.body;

	//This is the _id field for users in the mongodb database.
	const uniqueId: string = uuidv4();

	const userWithId = {
		_id: uniqueId,
		...user
	};

	dbConnection
		.collection('users')
		.insertOne(userWithId)
		.then(async (result: any) => {
			await sendMailForSigning(uniqueId, user.email);
			return res.status(201).json(result);
		})
		.catch((error: any) => {
			console.log(error);
			return res.status(500).json({ error: 'Could not post user' });
		});
};

const userExists = async (req: Request, res: Response) => {
	const dbConnection = getDb();
	const userEmail = req.body.email;
	try {
		dbConnection
			.collection('users')
			.findOne({ email: userEmail })
			.then((user: any) => {
				return res.status(200).json(!!user); // Returns true if user exists, false otherwise
			});
	} catch (error) {
		console.log(error);
		return res.status(500).send({ error: error });
	}
};

const getContractByUserId = async (req: Request, res: Response) => {
	const dbConnection = getDb();

	const userId = req.query.userId;

	const user = await dbConnection
		.collection('users')
		.findOne(
			{ _id: userId },
			{ projection: { 'contract.contract_pdf': 1 } }
		);

	if (user?.contract?.contract_pdf) {
		return res.status(200).json({ contract: user.contract.contract_pdf });
	} else {
		return res.status(500).json({ error: 'Could not get contract' });
	}
};

const getContractByUserEmail = async (req: Request, res: Response) => {
	const dbConnection = getDb();

	const userEmail = req.body.email;

	const user = await dbConnection
		.collection('users')
		.findOne(
			{ email: userEmail },
			{ projection: { 'contract.contract_pdf': 1 } }
		);

	if (user?.contract?.contract_pdf) {
		return res.status(200).json({ contract: user.contract.contract_pdf });
	} else {
		return res.status(500).json({ error: 'Could not get contract' });
	}
};

const updateContractByUserId = async (req: Request, res: Response) => {
	const dbConnection = getDb();

	const userId = req.body.userId;

	try {
		await dbConnection
			.collection('users')
			.updateOne(
				{ _id: userId },
				{
					$set: {
						'contract.contract_pdf': req.body.contract,
						//Sets the sign_date to the current date when conmtract is updated
						//In the format YYYY-MM-DD
						'contract.sign_date': new Date()
							.toISOString()				
							.split('T')[0]
					}
				}
			);

		return res.status(200).json({ message: 'User updated' });
	} catch (error) {
		return res.status(404).json({ message: 'User not found' });
	}
};

const isUserSigned = async (req: Request, res: Response) => {
	const dbConnection = getDb();
	const userId = req.body.userId;
	try {
		dbConnection
			.collection('users')
			.findOne({ _id: userId })
			.then((user: any) => {
				return res
					.status(200)
					.json((user && user.contract.sign_date) || !user); 
					// Returns true if user exists and is signed, false otherwise
			});
	} catch (error) {
		console.log(error);
		return res.status(500).send({ error: error });
	}
};

const resetContractByEmail = async (req: Request, res: Response) => {
	const dbConnection = getDb();

	const userEmail = req.body.email;
	const received_date = new Date().toISOString().split('T')[0];
	//Sets the received_date to the current date in the format YYYY-MM-DD

	try {
		await dbConnection
			.collection('users')
			.updateOne(
				{ email: userEmail },
				{ $set: { received_date: received_date, 
					'contract.sign_date': null, 
					'contract.contract_pdf': req.body.contract_pdf 
				} },
			);

		return res.status(200).json({ message: 'User updated' });
	} catch (error) {
		return res.status(404).json({ message: 'User not found' });
	}
};

const getAllSignedContractsFromGroup = async (req: Request, res: Response) => {
	const dbConnection = getDb();

	const group = req.body.group;

	try {
		const users = await dbConnection
			.collection('users')
			.find({
				'contract.sign_date': { $ne: null },
				'contract.group_slug': group.slug
			})
			.toArray();

		//Returns all signed contracts as base64 encoded strings
		//Returns the pdf along with the name and group slug for identifying the contract
		const contracts = users.map((user: any) => {
			return {
				contract: user.contract.contract_pdf,
				name: user.first_name + ' ' + user.last_name,
				group: user.contract.group_slug
			};
		});

		return res.status(200).json({ userContracts: contracts });
	} catch (error) {
		return res.status(404).json({ message: 'User not found' });
	}
};

const saveSignature = async (req: Request, res: Response) => {
	const dbConnection = getDb();

	//Gets the signature as a base64 encoded string
	const signature = req.body.signature;
	const userEmail = req.body.email;

	try {
		let signatureExists;
		dbConnection
			.collection('signatures')
			.findOne({ email: userEmail })
			.then((signatureObject: any) => {
				signatureExists = !!signatureObject; 
				// Returns true if user exists, false otherwise
				if (!signatureExists) {
					//If users do NOT have a signature insert a new one
					try {
						dbConnection
							.collection('signatures')
							.insertOne({
								email: userEmail,
								signature: signature
							});
						return res
							.status(200)
							.json({ message: 'Successfully saved signature' });
					} catch (error) {
						console.log(error);
						return res
							.status(500)
							.json({
								error: error,
								message: 'Could save signature'
							});
					}
				} else {
					//If users do have a signature update it 
					try {
						dbConnection
							.collection('signatures')
							.updateOne(
								{ email: userEmail },
								{ $set: { signature: signature } }
							);
						return res
							.status(200)
							.json({
								message: 'Successfully updated signature'
							});
					} catch (error) {
						console.log(error);
						return res
							.status(500)
							.json({
								error: error,
								message: 'Could not upload signature'
							});
					}
				}
			});
	} catch (error) {
		console.log(error);
		return res.status(500).send({ error: error });
	}
};

const getSignature = async (req: Request, res: Response) => {
	const dbConnection = getDb();

	const userEmail = req.body.email;

	try {
		dbConnection
			.collection('signatures')
			.findOne({ email: userEmail }, { _id: 0, signature: 1 })
			.then((signatureObject: any) => {
				if (!signatureObject)
					return res
						.status(200)
						.json({ message: 'No signature present' });
				return res
					.status(200)
					.json({ signature: signatureObject.signature });
			});
	} catch (error) {
		console.log(error);
		return res
			.status(500)
			.json({ error: error, message: 'Could not upload signature' });
	}
};

async function deleteUnsignedUsers() {
	const dbConnection = getDb();

	const oneYearAgo = new Date();
	oneYearAgo.setFullYear(oneYearAgo.getFullYear() - 1);

	await dbConnection.collection('users').deleteMany({
		received_date: { $lt: oneYearAgo.toISOString().substring(0, 10) },
		'contract.sign_date': { $eq: null }
	});
}

export {
	addUser,
	getContractByUserId,
	getContractByUserEmail,
	updateContractByUserId,
	userExists,
	resetContractByEmail,
	isUserSigned,
	getAllSignedContractsFromGroup,
	saveSignature,
	getSignature,
	deleteUnsignedUsers,
	getUsers,
	getLoggedInUser
};