import { IconInfoCircle, IconX } from '@tabler/icons-react';
import axios from 'axios';
import { useState } from 'react';
import JSZip from 'jszip';
import { ClipLoader } from 'react-spinners';
import Modal from 'react-modal';
import { Group } from '../../types/group';

//Button for downloading all signed contracts within a given group
const DownloadContractsButton = (props: { group: Group }) => {
	const [buttonInfoIsShown, setButtonInfoIsShown] = useState(false);
	const [isLoading, setIsLoading] = useState(false);
	const [errorWarningShown, setErrorWarningShown] = useState(false);

	const downloadContracts = () => {
		setIsLoading(true);
		axios
			.post(
				'/api/user/getAllSignedContractsFromGroup',
				{
					group: props.group
				},
				{
					headers: {
						'Content-Type': 'application/json'
					}
				}
			)
			.then((response) => {
				setIsLoading(false);
				let contractsWithNames = response.data.userContracts;

				const zip = new JSZip();

				// Decode base64 string to bytes
				contractsWithNames.forEach((contract: any) => {
					const binaryString = atob(contract.contract);

					// Convert binary to ArrayBuffer
					const arrayBuffer = new ArrayBuffer(binaryString.length);
					const uint8Array = new Uint8Array(arrayBuffer);
					for (let i = 0; i < binaryString.length; i++) {
						uint8Array[i] = binaryString.charCodeAt(i);
					}

					// Create Blob from ArrayBuffer
					const blob = new Blob([arrayBuffer], {
						type: 'application/octet-stream'
					});

					// Set the output file name
					const outputFileName = `Taushetserklæring ${
						contract.name
					} ${new Date().toISOString().split('T')[0]}.pdf`;

					zip.file(outputFileName, blob);
				});

				zip.generateAsync({ type: 'blob' }).then((blob) => {
					// Trigger download of the zip file
					const url = URL.createObjectURL(blob);
					const a = document.createElement('a');
					a.href = url;
					a.download = `${contractsWithNames[0].group}-contracts.zip`;
					a.click();
					URL.revokeObjectURL(url);
				});
			})
			.catch((error) => {
				setIsLoading(false);
				setErrorWarningShown(true);
				console.log(error);
			});
	};

	return (
		<>
			<div className='mr-3 flex flex-row items-center justify-center'>
				<div className='relative'>
					{buttonInfoIsShown && (
						<div className='absolute top-[-30px] right-[5px] z-50'>
							<div className='flex w-64 flex-col items-center justify-center rounded-md bg-custom-popup-dark p-1 text-center'>
								<p className='mb-1 flex items-center justify-center font-semibold text-white'>
									Her kan du laste ned alle signerte
									kontrakter
								</p>
							</div>
						</div>
					)}
				</div>
				<IconInfoCircle
					onMouseEnter={() => setButtonInfoIsShown(true)}
					onMouseLeave={() => setButtonInfoIsShown(false)}
					size={18}
					className='mr-1 text-white'
				/>
				<button
					onClick={downloadContracts}
					className='py px h-8 w-32 rounded bg-custom-blue text-xs text-white hover:bg-custom-light-blue sm:text-sm'
					data-cy="download-button">
					Last ned kontrakter
				</button>
			</div>
			{/*Loading Modal*/}
			<Modal
				style={{
					overlay: {
						backgroundColor: 'rgba(26,32,44,0.5)'
					}
				}}
				isOpen={isLoading}
				className='flex h-screen items-center justify-center'
				ariaHideApp={false}
				onRequestClose={() => setIsLoading(false)}>
				<div className='absolute'>
					<div className='mt-16 flex flex-col items-center justify-center'>
						<ClipLoader size={80} color='#FFFFFF' />
						<p className='mt-3em text-[2em] text-white'>
							Laster ned kontrakter...
						</p>
					</div>
				</div>
			</Modal>
			{/*Download Error Modal*/}
			<Modal
				style={{
					overlay: {
						backgroundColor: 'rgba(26,32,44,0.5)'
					}
				}}
				isOpen={errorWarningShown}
				className='flex h-screen items-center justify-center'
				ariaHideApp={false}
				onRequestClose={() => setErrorWarningShown(false)}>
				<div className='absolute'>
					<div className='z-50 flex w-4/5 flex-col rounded-md border border-gray-400 bg-custom-dark text-white'>
						<div className='flex justify-end'>
							<IconX
								size={30}
								onClick={() => setErrorWarningShown(false)}
								className='m-2 justify-self-end rounded-full hover:bg-gray-700'
							/>
						</div>
						<p className='mt-3em mb-6 text-center text-[2em] text-white'>
							Kunne ikke laste ned kontrakter
						</p>
					</div>
				</div>
			</Modal>
		</>
	);
};

export { DownloadContractsButton };
