import {
	IconSearch,
	IconChevronDown,
	IconAlertCircle
} from '@tabler/icons-react';
import { useEffect, useRef, useState } from 'react';
import { Group } from '../../types/group';
import axios from 'axios';

//Dropdownmenu for selecting a group to filter users by
const GroupSelector = (props: { setGroupFilter: any; group: Group }) => {
	const [currentGroup, setCurrentGroup] = useState<Group>(props.group);
	const [groups, setGroups] = useState<Group[]>([]);
	const [shownGroups, setShownGroups] = useState<Group[]>([]);
	const [groupsVisible, setGroupsVisible] = useState(false);
	const [groupSearch, setGroupSearch] = useState(currentGroup.name);
	const [groupWarningVisible, setGroupWarningVisible] = useState(false);
	const ref = useRef<HTMLDivElement>(null);

	useEffect(() => {
		props.setGroupFilter(currentGroup);
	}, [currentGroup]);

	//Runs every time component is rendered
	useEffect(() => {
		function handleClickOutside(event: MouseEvent): void {
			if (ref.current && !ref.current.contains(event.target as Node)) {
				setGroupsVisible(false);
			}
		}
		document.addEventListener('mousedown', handleClickOutside);
		return () => {
			document.removeEventListener('mousedown', handleClickOutside);
		};
	});

	//Runs only on the initial render
	useEffect(() => {
		fetchGroups().then((groups) => {
			const group = groups[0];
			setCurrentGroup(group);
			setGroupSearch(group.name);
		});
	}, []);

	const fetchGroups = async (): Promise<Group[]> => {
		return new Promise((resolve, reject) => {
			axios
				.get('/api/group/groups')
				.then((response) => {
					let groups: Group[] = response.data.groups.map(
						(group: any) => {
							return {
								name: group.name,
								slug: group.slug
							};
						}
					);
					setGroups(groups);
					setShownGroups(groups);
					resolve(groups);
				})
				.catch((error) => {
					reject(error);
				});
		});
	};

	const sortGroups = (groups: Group[]) => {
		return groups.sort((a, b) => a.name.localeCompare(b.name));
	};

	//Maps the groups into displayable divs
	const createGroupDivs = (groups: Group[]) => {
		const sortedGroups = sortGroups(groups);
		return sortedGroups.map((group) => (
			<div
				onClick={() => {
					setCurrentGroup(group);
					setGroupSearch(group.name);
					setGroupsVisible(false);
					setGroupWarningVisible(false);
				}}
				tabIndex={0}
				key={group.slug}
				className='p-2 flex justify-center items-center hover:bg-slate-300 focus:bg-slate-300'>
				<p>{group.name}</p>
			</div>
		));
	};

	//Filters the list to find groupnames matching the search
	const filterGroupList = (query: string) => {
		const re = RegExp(`.*${query.toLowerCase().split('').join('.*')}.*`);
		const matches = groups.filter((group) =>
			group.name.toLowerCase().match(re)
		);
		return matches;
	};

	const onSearchInput = (query: string) => {
		setGroupWarningVisible(false);
		setGroupSearch(query);
		setGroupsVisible(true);
		let matchingGroups = filterGroupList(groupSearch);
		setShownGroups(matchingGroups);
		if (query === '') setShownGroups(groups);
		if (groupIsValid(query)) {
			const group = getGroupObject(query);
			if (group) setCurrentGroup(group);
		}
	};

	//Checks if the groupName provided is an existing group
	const groupIsValid = (groupName: string) => {
		let group = formatGroupName(groupName);
		let groupNames = groups.map((group: Group) => group.name);
		return groupNames.includes(group);
	};

	const formatGroupName = (groupName: string) => {
		let group = groupName.trim();
		return group.charAt(0).toUpperCase() + group.slice(1);
	};

	//Takes the name of the group and returns the
	const getGroupObject = (groupName: string) => {
		return groups.find((group) => {
			return (group.name = groupName);
		});
	};

	const handleKeyPress = (event: React.KeyboardEvent) => {
		if (event.key === 'Enter') {
			if (groupIsValid(groupSearch)) {
				setGroupsVisible(false);
				let preppedGroup = formatGroupName(groupSearch);
				let group = getGroupObject(preppedGroup);
				if (group) setCurrentGroup(group);
			} else {
				setGroupWarningVisible(true);
			}
		}
	};

	return (
		<>
			{groups.length > 1 && (
				<div ref={ref} className='relative'>
					{groupWarningVisible && (
						<div className='absolute top-[-2em] w-full'>
							<div className='flex justify-center items-center'>
								<IconAlertCircle
									size={18}
									className='stroke-custom-red mr-2'
								/>
								<p className='text-custom-red' data-cy='group-warning'>
									{groupSearch.length === 0
										? 'Velg en gruppe'
										: 'Gruppen eksisterer ikke'}
								</p>
							</div>
						</div>
					)}
					<div className='flex bg-custom-dark ml-2 h-full border rounded justify-center items-center'>
						<IconSearch size={18} className='mx-1 stroke-white' />
						<input
							type='text'
							value={groupSearch}
							onChange={(search) =>
								onSearchInput(search.target.value)
							}
							onKeyDown={(e) => {
								handleKeyPress(e);
							}}
							placeholder='Gruppe'
							className='outline-none w-40 bg-custom-dark text-white'
							data-cy='group-input'
						/>
						<IconChevronDown
								size={18}
								onClick={() => setGroupsVisible(!groupsVisible)}
								className='mx-1 stroke-white cursor-pointer'
								data-cy='group-list-arrow'
							/>
					</div>
					{groupsVisible && (
						<div className='bg-white rounded absolute left-1 top-8 w-full py-1 overflow-y-auto max-h-72 z-50'
							 data-cy='group-list'>
							{createGroupDivs(shownGroups)}
						</div>
					)}
				</div>
			)}
		</>
	);
};

export default GroupSelector;
