import axios from 'axios';
import { useContext, ChangeEvent, useEffect, useRef, useState } from 'react';
import { useNavigate } from 'react-router-dom';
import { UserContext, UserContextProps } from '../../contexts/UserContextProps';
import Modal from 'react-modal';
import { IconX } from '@tabler/icons-react';
import { ClipLoader } from 'react-spinners';
import Sign from '../Functionalities/Sign';

//Profile display showing user information, and various user-actions
const ProfileView = () => {
	const [signature, setSignature] = useState('');
	const [profileOpen, setProfileOpen] = useState(false);
	const [uploadWarningOpen, setUploadWarningOpen] = useState(false);
	const [uploadConfirmationOpen, setUploadConfirmationOpen] = useState(false);
	const [signatureUploading, setSignatureUploading] = useState(false);
	const [signatureDownloading, setSignatureDownloading] = useState(false);
	const [signatureModalOpen, setSignatureModalOpen] = useState(false);
	const [signatureCanvasOpen, setSignatureCanvasOpen] = useState(false);
	const signatureInputRef = useRef<HTMLInputElement>(null);
	const ref = useRef<HTMLDivElement>(null);
	const { userInfo } = useContext(UserContext) as UserContextProps;
	const navigate = useNavigate();

	useEffect(() => {
		function handleClickOutside(event: MouseEvent): void {
			if (ref.current && !ref.current.contains(event.target as Node)) {
				setProfileOpen(false);
			}
		}
		document.addEventListener('mousedown', handleClickOutside);
		return () => {
			document.removeEventListener('mousedown', handleClickOutside);
		};
	});

	const logout = async () => {
		await axios
			.get('/api/auth/logout')
			.then(() => {
				navigate('/login');
			})
			.catch((err) => {
				console.log('Something went wrong while logging out');
				console.log(err);
				navigate('/login');
			});
	};

	const openSignatureInput = () => {
		if (signatureInputRef.current) {
			signatureInputRef.current.click();
		}
	};

	const handleFileInputChange = (e: ChangeEvent<HTMLInputElement>) => {
		const selectedfile = e.target.files;
		if (selectedfile != undefined && selectedfile.length > 0) {
			const [imageFile] = selectedfile;
			const fileReader = new FileReader();
			fileReader.onload = () => {
				const srcData = fileReader.result;
				setSignatureUploading(true);
				axios
					.put(
						'/api/user/updateSignature',
						{
							email: userInfo.email,
							signature: srcData?.toString()
						},
						{
							headers: {
								'Content-Type': 'application/json'
							}
						}
					)
					.then(() => {
						setSignatureUploading(false);
						setProfileOpen(false);
						setUploadConfirmationOpen(true);
						setSignature(srcData?.toString()!);
					})
					.catch((error) => {
						console.log('Could not update signature');
						console.log(error);
						setUploadWarningOpen(true);
						setSignatureUploading(false);
					});
			};
			fileReader.readAsDataURL(imageFile);
		}
	};

	const getSignature = async () => {
		setSignatureDownloading(true);
		axios
			.post(
				'/api/user/getSignature',
				{
					email: userInfo.email
				},
				{
					headers: {
						'Content-Type': 'application/json'
					}
				}
			)
			.then((response) => {
				const signature = response.data.signature;
				if (signature) setSignature(signature);
				setSignatureDownloading(false);
			})
			.catch((error) => {
				console.log('Could not get signature');
				console.log(error);
				setSignatureDownloading(false);
			});
	};

	const showSignatureOnClick = () => {
		setProfileOpen(false);
		setSignatureModalOpen(true);
	};

	const userHasSignature = () => {
		return !!signature;
	};

	const signatureCanvasOnSave = (dataURL: string) => {
		setSignatureUploading(true);
		axios
			.put(
				'/api/user/updateSignature',
				{
					email: userInfo.email,
					signature: dataURL
				},
				{
					headers: {
						'Content-Type': 'application/json'
					}
				}
			)
			.then(() => {
				setSignatureUploading(false);
				setProfileOpen(false);
				setUploadConfirmationOpen(true);
				setSignature(dataURL);
			})
			.catch((error) => {
				console.log('Could not update signature');
				console.log(error);
				setUploadWarningOpen(true);
				setSignatureUploading(false);
			});
	};

	return (
		<div ref={ref} className='relative'>
			<div
				onClick={() => setProfileOpen(!profileOpen)}
				className='mt-5 mr-3 flex cursor-pointer flex-col items-center rounded-lg border p-2 md:mr-20'
				data-cy="profile-view-button">
				<p className='text-lg text-white'>
					{userInfo.first_name + ' ' + userInfo.last_name}
				</p>
			</div>
			{profileOpen && (
				<div className='absolute right-[200px] top-[30px] z-50 w-80 rounded-md bg-neutral-700 text-white' data-cy='profile-view'>
					<div className='flex flex-col items-center justify-center'>
						<div className='flex w-full flex-col items-center justify-center rounded-t-md bg-neutral-800'>
							<p className='mt-3'>
								{userInfo.first_name + ' ' + userInfo.last_name}
							</p>
							<p className='mb-3 text-sm text-gray-400'>
								{userInfo.email}
							</p>
						</div>
						<div className='flex w-full flex-col items-center justify-center border-b border-t'>
							<button
								onClick={openSignatureInput}
								className='my-1 w-full bg-neutral-700 p-1 hover:brightness-125'
								data-cy='signature-upload'>
								Last opp ny signatur
							</button>
							<input
								type='file'
								onChange={handleFileInputChange}
								ref={signatureInputRef}
								className='hidden'
								data-cy='signature-file-input'
							/>
							<button
								onClick={() => {
									setSignatureCanvasOpen(true);
									setProfileOpen(false);
								}}
								className='my-1 w-full bg-neutral-700 p-1 hover:brightness-125'
								data-cy='signature-draw-upload'>
								Tegn ny signatur
							</button>
							<button
								onClick={showSignatureOnClick}
								className='my-1 w-full bg-neutral-700 p-1 hover:brightness-125'
								data-cy='signature-view-button'>
								Se signatur
							</button>
						</div>
						<button
							onClick={logout}
							className='my-2 w-32 rounded-md bg-custom-blue p-2 hover:brightness-125'
							data-cy='logout-button'>
							Logg ut
						</button>
					</div>
				</div>
			)}
			<Sign
				isOpen={signatureCanvasOpen}
				setIsOpen={setSignatureCanvasOpen}
				onSave={signatureCanvasOnSave}
			/>
			<Modal
				style={{
					overlay: {
						backgroundColor: 'rgba(26,32,44,0.5)'
					}
				}}
				isOpen={signatureModalOpen}
				className='z-50 flex h-screen items-center justify-center'
				ariaHideApp={false}
				onAfterOpen={() => getSignature()}
				onRequestClose={() => setSignatureModalOpen(false)}>
				<div className='flex w-max flex-col items-center justify-center rounded-md border border-gray-400 bg-custom-dark text-white'>
					<div className='flex w-full justify-end'>
						<IconX
							size={30}
							onClick={() => setSignatureModalOpen(false)}
							className='m-2 justify-self-end rounded-full hover:bg-gray-700'
						/>
					</div>
					{!signatureDownloading &&
						(userHasSignature() ? (
							<img
								src={signature}
								alt='Signatur'
								className='mb-10 h-[10em] w-[40em] object-contain'
								data-cy='signature-view'
							/>
						) : (
							<p className='px-10 pb-10 text-[1.5em] font-bold' data-cy='signature-view-empty'>
								Ingen signatur
							</p>
						))}
					{signatureDownloading && (
						<div className='m-5 flex flex-col items-center justify-center p-10'
							 data-cy='signature-download-loading'>
							<ClipLoader size={80} color='#FFFFFF' />
							<p className='mt-3em text-[2em] text-white'>
								Laster ned signatur...
							</p>
						</div>
					)}
				</div>
			</Modal>
			<Modal
				style={{
					overlay: {
						backgroundColor: 'rgba(26,32,44,0.5)'
					}
				}}
				isOpen={signatureUploading}
				className='flex h-screen items-center justify-center'
				ariaHideApp={false}
				onRequestClose={() => setSignatureUploading(false)}>
				<div className='absolute'>
					<div className='mt-16 flex flex-col items-center justify-center'
						 data-cy='signature-upload-loading'>
						<ClipLoader size={80} color='#FFFFFF' />
						<p className='mt-3em text-[2em] text-white'>
							Laster opp signatur...
						</p>
					</div>
				</div>
			</Modal>
			<Modal
				style={{
					overlay: {
						backgroundColor: 'rgba(26,32,44,0.5)'
					}
				}}
				isOpen={uploadWarningOpen}
				className='flex h-screen items-center justify-center'
				ariaHideApp={false}
				onRequestClose={() => setUploadWarningOpen(false)}>
				<div className='absolute'
					 data-cy='signature-upload-error'>
					<div className='z-50 flex w-4/5 flex-col rounded-md border border-gray-400 bg-custom-dark text-white'>
						<div className='flex justify-end'>
							<IconX
								size={30}
								onClick={() => setUploadWarningOpen(false)}
								className='m-2 justify-self-end rounded-full hover:bg-gray-700'
								data-cy='close-signature-upload-error'
							/>
						</div>
						<p className='mt-3em mb-6 text-center text-[2em] text-white'>
							Kunne ikke laste opp signatur
						</p>
					</div>
				</div>
			</Modal>
			<Modal
				style={{
					overlay: {
						backgroundColor: 'rgba(26,32,44,0.5)'
					}
				}}
				isOpen={uploadConfirmationOpen}
				className='flex h-screen items-center justify-center'
				ariaHideApp={false}
				onRequestClose={() => setUploadConfirmationOpen(false)}>
				<div className='absolute'
					 data-cy='signature-upload-confirmation'>
					<div className='z-50 flex w-4/5 flex-col rounded-md border border-gray-400 bg-custom-dark text-white'>
						<div className='flex justify-end'>
							<IconX
								size={30}
								onClick={() => setUploadConfirmationOpen(false)}
								className='m-2 justify-self-end rounded-full hover:bg-gray-700'
								data-cy='close-signature-upload-confirmation'
							/>
						</div>
						<p className='mt-3em mb-6 px-20 text-center text-[2em] text-white'>
							Signatur lastet opp
						</p>
						<img
							src={signature}
							alt='Signatur'
							className='mb-10 h-[10em] w-[40em] object-contain'
						/>
					</div>
				</div>
			</Modal>
		</div>
	);
};

export default ProfileView;
