import {
	IconCircleCheck,
	IconAlertTriangle,
	IconAlertOctagon
} from '@tabler/icons-react';
import { useContext, useRef, useState } from 'react';
import Avatar from 'react-avatar';
import { useNavigate } from 'react-router-dom';
import Modal from 'react-modal';
import { IconX } from '@tabler/icons-react';
import { User } from '../../types/user';
import { Group } from '../../types/group';
import { getDataUri } from '../../utils/getBase64Uri';
import axios from 'axios';
import { ClipLoader } from 'react-spinners';
import { UserContext, UserContextProps } from '../../contexts/UserContextProps';

//Row displaying a User
const UserRow = (props: { user: User; group: Group }) => {
	
	let [contractShown, setContractShown] = useState(false);
	let [contract, setContract] = useState<string>('');
	let [isLoading, setIsLoading] = useState(false);
	const ndaPdf = useRef<HTMLIFrameElement>(null);
	let displayButton = useRef(null);
	const { userInfo } = useContext(UserContext) as UserContextProps;
	const navigate = useNavigate();

	let user = props.user;

	const sendNDAOnClick = () => {
		navigate('/send-nda', {
			state: {
				user: user,
				group: props.group
			}
		});
	};

	const showSignStatusIcon = () => {
		if (user.contract.sign_date) {
			return <IconCircleCheck size='40' className='text-custom-green' />;
		} else if (user.received_date && !user.contract.sign_date) {
			return <IconAlertOctagon size='40' className='text-[#ffd000]' />;
		} else {
			return <IconAlertTriangle size='40' className='text-custom-red' />;
		}
	};

	const showSignStatusText = () => {
		if (user.contract.sign_date) {
			return 'Signert';
		} else if (user.received_date && !user.contract.sign_date) {
			return 'Kontrakt sendt';
		} else {
			return 'Usignert';
		}
	};

	const getDateString = (date: Date | null) => {
		if (date)
			return (
				date.getDate() +
				'.' +
				(date.getMonth() + 1) +
				'.' +
				date.getFullYear()
			);
		return '';
	};

	const getContractByEmail = async (email: string) => {
		try {
			setIsLoading(true);
			await axios
				.post(
					'/api/user/getContractByUserEmail',
					{
						email: email
					},
					{
						headers: {
							'Content-Type': 'application/json'
						}
					}
				)
				.then((response) => {
					setContract(response.data.contract);
					setIsLoading(false);
				});
		} catch (error) {
			setIsLoading(false);
			console.log(error);
		}
	};

	const reloadFrame = () => {
		if (ndaPdf.current && ndaPdf.current.contentWindow) {
			ndaPdf.current.contentWindow.location.reload();
		}
	};

	const userIsSelf = (user: User) => {
		return user.email !== userInfo.email;
	};

	return (
		<>
			<div className='min-h-28 flex flex-col items-center justify-center border border-custom-yellow text-white md:grid md:grid-cols-[2fr,3fr] lg:grid lg:grid-cols-[250px,5fr,5fr,200px]'
				 data-cy='user-row'>
				<div className='my-3 ml-4 flex flex-row justify-start'>
					<Avatar
						name={`${user.first_name} ${user.last_name}`}
						src={user.picture_url ? user.picture_url : ''}
						size={'52'}
						className='mr-5 aspect-square rounded-full object-cover'
					/>
					<div className='flex flex-col'>
						<p className='font-bold' data-cy='user-name'>{`${user.first_name} ${user.last_name}`}</p>
						<p>{user.email}</p>
					</div>
				</div>
				<button
					hidden={!user.contract.sign_date}
					onClick={() => setContractShown(true)}
					className='h-7 w-52 rounded-sm bg-green-600 text-center font-semibold hover:bg-custom-green md:ml-20 md:mb-5'
					data-cy='view-contract-button'>
					Vis taushetserklæring
				</button>
				{/*Renders an empty div when button is hided to make it all line up well*/}
				{!userIsSelf(user) && <div className='w-100'></div>}
				{userIsSelf(user) && (
					<>
						<button
							hidden={Boolean(
								!(
									user.received_date &&
									!user.contract.sign_date
								) && !user.contract.sign_date
							)}
							onClick={sendNDAOnClick}
							className='h-7 w-52 rounded-sm bg-[#ffd000] text-center text-sm font-semibold hover:brightness-90 md:ml-20 md:mb-5'
							data-cy='generate-new-contract-button'>
							Generer ny taushetserklæring
						</button>
						<button
							hidden={Boolean(user.received_date)}
							ref={displayButton}
							onClick={sendNDAOnClick}
							className='h-7 w-52 rounded-sm bg-custom-red text-center font-semibold hover:bg-red-600 md:ml-20 md:mb-5'
							data-cy='generate-contract-button'>
							Generer taushetserklæring
						</button>
					</>
				)}
				{/*Fills in one of the columns with an empty div to align the buttons properly*/}
				{!user.contract.sign_date && <div className='w-100'></div>}
				<div className='my-3 flex w-20 flex-row items-center justify-center md:ml-6 md:justify-start lg:flex-col lg:items-start'>
					<div className='flex-column flex items-center'>
						<div className='mr-1'>{showSignStatusIcon()}</div>
						<p className='mr-2'>{showSignStatusText()}</p>
					</div>
					<div className='flex items-center justify-center'>
						<p hidden={!user.contract.sign_date} className='ml-3'>
							{getDateString(
								user.contract.sign_date
									? user.contract.sign_date
									: null
							)}
						</p>
						<p
							hidden={
								!(
									user.received_date &&
									!user.contract.sign_date
								)
							}
							className='ml-3'>
							{getDateString(
								user.received_date ? user.received_date : null
							)}
						</p>
					</div>
				</div>
			</div>
			<Modal
				style={{
					overlay: {
						backgroundColor: 'rgba(26,32,44,0.5)'
					}
				}}
				isOpen={contractShown}
				className='flex h-screen items-center justify-center'
				ariaHideApp={false}
				onAfterOpen={() => {
					getContractByEmail(user.email);
					reloadFrame();
				}}
				onRequestClose={() => setContractShown(false)}>
				{isLoading && (
					<div className='mt-36 flex flex-col items-center justify-center' data-cy='contract-loading'>
						<ClipLoader size={80} color='#FFFFFF' />
						<p className='mt-3em text-[2em] text-white'>
							Kontrakt lastes inn...
						</p>
					</div>
				)}
				{!isLoading && (
					<div className='z-50 flex w-4/5 flex-col rounded-md border border-gray-400 bg-custom-dark text-white'>
						<div className='flex justify-end'>
							<IconX
								size={30}
								onClick={() => setContractShown(false)}
								className='m-2 justify-self-end rounded-full hover:bg-gray-700'
								data-cy='close-contract-view'
							/>
						</div>
						<iframe
							ref={ndaPdf}
							className='h-128 w-full'
							src={getDataUri(contract)}
							data-cy='contract-view'
						/>
					</div>
				)}
			</Modal>
		</>
	);
};

export default UserRow;
