import UserList from '../components/Homepage/UserList';
import TopBar from '../components/Homepage/TopBar';
import Header from '../components/Global/Header';
import ProfileView from '../components/Homepage/ProfileView';
import { useEffect, useState, useContext } from 'react';
import axios from 'axios';
import { User } from '../types/user';
import { Group } from '../types/group';
import { useFirstRender } from '../hooks/useFirstRender';
import { UserContext, UserContextProps } from '../contexts/UserContextProps';

const HomePage = () => {
	const [users, setUsers] = useState<User[]>([]);
	const [shownUsers, setShownUsers] = useState(users);
	const [usersAreLoading, setUsersAreLoading] = useState(true);
	const [groupForFilter, setGroupFilter] = useState<Group>({
		group_id: -1,
		name: '',
		slug: ''
	});
	//Filter year 0 means show all years
	const [yearForFilter, setYearForFilter] = useState<number>(0);
	const [showSignedUsers, setShowSignedUsers] = useState<boolean>(true);
	const { setUserInfo } = useContext(UserContext) as UserContextProps;

	useEffect(() => {
		setShownUsers(filterUsers(users));
	}, [showSignedUsers, yearForFilter]);

	let userIsInHS = false;
	useEffect(() => {
		getUserInfo();
	}, []);

	//Fetch users every time group-filter is updated but not on first render
	const firstRender = useFirstRender();
	useEffect(() => {
		if (!firstRender) {
			fetchUsers();
		}
	}, [groupForFilter]);

	//Runs only on the initial render
	useEffect(() => {
		fetchUsers();
		getUserInfo();
	}, []);

	//Gets the userinfo of the logged in user and stores it in the global UserContext
	const getUserInfo = async () => {
		await axios.get('/api/user/loggedInUser').then((response) => {
			let user = response.data.user;
			if (
				user.memberships.some(
					(membership: { slug: string }) =>
						membership.slug == 'hovedstyret'
				)
			) {
				userIsInHS = true;
			}
			const limUserInfo = {
				first_name: user.first_name,
				last_name: user.last_name,
				email: user.contact_email,
				ntnuiNo: user.ntnui_no,
				memberships: user.memberships,
				userInHS: userIsInHS
			};
			setUserInfo(limUserInfo);
		});
	};

	const fetchUsers = async () => {
		setUsersAreLoading(true);
		await axios
			.get(`/api/user/users?groupslug=${groupForFilter.slug}`)
			.then((response) => {
				let users: User[] = response.data.users.map((user: User) => {
					if (user.contract) {
						return {
							first_name: user.first_name,
							last_name: user.last_name,
							email: user.email,
							received_date: user.received_date
								? new Date(user.received_date)
								: null,
							picture_url: user.picture_url,
							type: user.type,
							contract: {
								group_slug: user.contract.group_slug,
								sign_date: user.contract.sign_date
									? new Date(user.contract.sign_date)
									: null
							}
						};
					} else {
						return {
							first_name: user.first_name,
							last_name: user.last_name,
							email: user.email,
							received_date: null,
							picture_url: user.picture_url,
							type: user.type,
							contract: {
								group_slug: null,
								sign_date: null
							}
						};
					}
				});
				setUsers(users);
				setUsersAreLoading(false);
				setShownUsers(filterUsers(users));
			});
	};

	const filterUsers = (users: User[]) => {
		let usersFiltered = users
			.filter((user: User) => {
				if (!showSignedUsers) {
					if (!user.contract.sign_date) return user;
				} else {
					return user;
				}
			})
			.filter((user: User) => {
				if (yearForFilter !== 0) {
					if (!user.contract.sign_date) return false;
					const userSignYear = user.contract.sign_date.getFullYear();
					if (userSignYear === yearForFilter) return user;
				} else {
					return user;
				}
			});
		return usersFiltered;
	};

	return (
		<>
			<div className='ml-5 flex justify-between'>
				<Header />
				<ProfileView />
			</div>
			<div className='mx-3 my-5 md:mx-20' data-cy='home-page'>
				<TopBar
					users={users}
					setUsers={setShownUsers}
					setGroupFilter={setGroupFilter}
					group={groupForFilter}
					setYearForFilter={setYearForFilter}
					year={yearForFilter}
					setShowSignedUsers={setShowSignedUsers}
					showSignedUsers={showSignedUsers}
				/>
				<UserList
					users={shownUsers}
					isLoading={usersAreLoading}
					group={groupForFilter}
				/>
			</div>
		</>
	);
};

export default HomePage;
