import { useEffect, useRef, useState } from 'react';
import Header from '../components/Global/Header';
import Sign from '../components/Functionalities/Sign';
import { IconSignature, IconPencil } from '@tabler/icons-react';
import axios from 'axios';
import { useNavigate, useParams } from 'react-router-dom';
import { useFirstRender } from '../hooks/useFirstRender';

//SignaturePage shown for users receiving a contract
const ReceiverSignaturePage = () => {
	const [isOpen, setIsOpen] = useState(false);
	const [hasUploaded, setHasUploaded] = useState(false);
	const [contract, setContract] = useState<string>('');
	const ndaPdf = useRef<HTMLIFrameElement>(null);
	const params = useParams();
	const userId = params.userid;
	const navigate = useNavigate();
	if (!userId) navigate('not-found');

	const firstRender = useFirstRender();
	useEffect(() => {
		//Checks if user exists and if user has signed
		axios
			.post(
				'/api/user/isUserSigned',
				{
					userId: userId
				},
				{
					headers: {
						'Content-Type': 'application/json'
					}
				}
			)
			.then((response) => {
				const userExistsAndIsNotSigned = response.data;
				if (userExistsAndIsNotSigned) {
					navigate('not-found');
				}
			})
			.catch((error) => {
				navigate('not-found');
				console.log(error);
			});
		if (firstRender) getContractByUserId(userId!);
	}, []);

	const reloadFrame = () => {
		if (ndaPdf.current && ndaPdf.current.contentWindow) {
			ndaPdf.current.contentWindow.location.reload();
		}
	};

	const getContractByUserId = async (userId: string | null) => {
		try {
			if (!userId) throw Error('Could not get userId');
			await axios
				.get('/api/user/getContractByUserId', { params: { userId: userId } })
				.then((response) => {
					setContract(response.data.contract);
					setHasUploaded(true);
					reloadFrame();
				});
		} catch (error) {
			console.log(error);
		}
	};

	const getEditedContract = async (signature: string | null | undefined) => {
		try {
			await axios
				.post(
					'/api/pdf/editContractForReceiver',
					{
						contract: contract,
						signature: signature
					},
					{
						headers: {
							'Content-Type': 'application/json'
						}
					}
				)
				.then((response) => {
					setContract(response.data.pdf);
					setHasUploaded(true);
					reloadFrame();
				});
		} catch (error) {
			console.log(error);
		}
	};

	const sendEmail = () => {
		axios.put('/api/user/updateContract', {
			userId: userId,
			contract: contract
		}),
			{
				headers: {
					'Content-Type': 'application/json'
				}
			};
		axios.post('/api/mail/sendEmail', {
			userId: userId,
			contract: contract
		}),
			{
				headers: {
					'Content-Type': 'application/json'
				}
			};
	};

	const uploadSignature = (e: React.ChangeEvent<HTMLInputElement>) => {
		const selectedfile = e.target.files;
		if (selectedfile != undefined && selectedfile.length > 0) {
			const [imageFile] = selectedfile;
			const fileReader = new FileReader();
			fileReader.onload = () => {
				const srcData = fileReader.result;
				getEditedContract(srcData?.toString());
			};
			fileReader.readAsDataURL(imageFile);
		}
		setHasUploaded(true);
	};

	const getDataUri = (base64String: string) => {
		return `data:application/pdf;base64,${base64String}`;
	};

	const onSend = () => {
		sendEmail();
		navigate('/confirmation');
	};

	const signatureCanvasOnSave = (dataURL: string) => {
		getEditedContract(dataURL);
		setHasUploaded(true);
	};

	return (
		<div>
			<Header />
			<div className='mx-3 my-5 mt-20 flex flex-col items-center sm:mx-20 sm:my-10 sm:items-end'>
				<iframe
					className='my-3 h-128 w-full'
					src={getDataUri(contract)}
					ref={ndaPdf}
					data-cy='contract-iframe'
				/>
				<div className='flex'>
					<div className='flex rounded-sm border border-gray-400'>
						<label
							htmlFor='fileUpload'
							className='flex grow items-center justify-center border-custom-blue bg-custom-blue px-3 py-1 text-center text-xl font-semibold text-white hover:bg-custom-light-blue'
							data-cy='signature-file-input-button'>
							{hasUploaded
								? 'Signatur lastet opp'
								: 'Last opp signatur'}
						</label>
						<input
							type='file'
							id='fileUpload'
							onChange={(e) => uploadSignature(e)}
							className='hidden'
							data-cy='signature-file-input'
						/>
						<div
							onClick={() => setIsOpen(true)}
							className='px-3 py-1 text-center text-sm font-semibold text-white hover:bg-gray-700'>
							{'Eller tegn'}
							<div className='flex flex-row items-center justify-center'
								 data-cy='signature-draw-button'>
								<IconSignature size={26} />
								<IconPencil size={26} />
							</div>
						</div>
					</div>
					<button
						onClick={onSend}
						disabled={!hasUploaded}
						style={{
							backgroundColor: hasUploaded ? '#39c16c' : ''
						}}
						className='ml-5 rounded-sm bg-gray-500 px-3 py-1 text-center text-xl font-semibold text-white hover:brightness-125'
						data-cy='send-contract-button'>
						Send
					</button>
				</div>
				<Sign
					isOpen={isOpen}
					setIsOpen={setIsOpen}
					onSave={signatureCanvasOnSave}
				/>
			</div>
		</div>
	);
};

export default ReceiverSignaturePage;
