import { useContext, useEffect, useRef, useState } from 'react';
import Header from '../components/Global/Header';
import Sign from '../components/Functionalities/Sign';
import { IconSignature, IconPencil } from '@tabler/icons-react';
import axios from 'axios';
import { useLocation, useNavigate } from 'react-router-dom';
import { User } from '../types/user';
import { Group } from '../types/group';
import { getDataUri } from '../utils/getBase64Uri';
import { UserContext, UserContextProps } from '../contexts/UserContextProps';

//SignaturePage shown for users sending a contract
const SenderSignaturePage = () => {
	const [isOpen, setIsOpen] = useState(false);
	const [hasUploaded, setHasUploaded] = useState(false);
	const [contract, setContract] = useState<string>('');
	const ndaPdf = useRef<HTMLIFrameElement>(null);
	const { userInfo } = useContext(UserContext) as UserContextProps;
	const navigate = useNavigate();
	const { state } = useLocation();
	const user: User = state.user;
	const group: Group = state.group;

	useEffect(() => {
		axios
			.post(
				'/api/user/getSignature',
				{
					email: userInfo.email
				},
				{
					headers: {
						'Content-Type': 'application/json'
					}
				}
			)
			.then((response) => {
				const signature = response.data.signature;
				if (signature) {
					getEditedContract(signature);
					setHasUploaded(true);
				} else {
					getEditedContract(null);
				}
			})
			.catch((error) => {
				console.log('Could not get signature');
				console.log(error);
			});
	}, []);

	const reloadFrame = () => {
		if (ndaPdf.current && ndaPdf.current.contentWindow) {
			ndaPdf.current.contentWindow.location.reload();
		}
	};

	const getEditedContract = async (signature: string | null | undefined) => {
		try {
			await axios
				.post(
					'/api/pdf/editContract',
					{
						contract: contract,
						signature: signature,
						first_name: user.first_name,
						last_name: user.last_name,
						email: user.email,
						type: user.type,
						group: group
					},
					{
						headers: {
							'Content-Type': 'application/json'
						}
					}
				)
				.then((response) => {
					setContract(response.data.pdf);
					reloadFrame();
				});
		} catch (error) {
			console.log(error);
		}
	};

	const sendEmail = async () => {
		const userExists = (
			await axios.post(
				'/api/user/doesUserExist',
				{
					email: user.email
				},
				{
					headers: {
						'Content-Type': 'application/json'
					}
				}
			)
		).data;

		if (!userExists) {
			axios
				.post(
					'/api/user/addUser',
					{
						first_name: user.first_name,
						last_name: user.last_name,
						email: user.email,
						received_date: new Date().toISOString().split('T')[0],
						picture_url: user.picture_url,
						type: user.type,
						contract: {
							group_slug: group.slug,
							contract_pdf: contract,
							sign_date: null
						}
					},
					{
						headers: {
							'Content-Type': 'application/json'
						}
					}
				)
				.catch((error) => {
					console.log('Could not add user');
					console.log(error);
				});
		} else {
			axios
				.put(
					'/api/user/resetContractByEmail',
					{
						email: user.email,
						contract_pdf: contract
					},
					{
						headers: {
							'Content-Type': 'application/json'
						}
					}
				)
				.catch((error) => {
					console.log('Could not update received_date');
					console.log(error);
				});
		}
	};

	const uploadSignature = (e: React.ChangeEvent<HTMLInputElement>) => {
		const selectedfile = e.target.files;
		if (selectedfile != undefined && selectedfile.length > 0) {
			const [imageFile] = selectedfile;
			const fileReader = new FileReader();
			fileReader.onload = () => {
				const srcData = fileReader.result;
				getEditedContract(srcData?.toString());
			};
			fileReader.readAsDataURL(imageFile);
		}
		setHasUploaded(true);
	};

	const onSend = () => {
		sendEmail();
		navigate('/confirmation', {
			state: {
				user: user
			}
		});
	};

	const signatureCanvasOnSave = (dataURL: string) => {
		getEditedContract(dataURL);
		setHasUploaded(true);
	};

	return (
		<div>
			<Header />
			<div className='mx-3 my-5 mt-20 flex flex-col items-center sm:mx-20 sm:my-10 sm:items-end'>
				<iframe
					className='my-3 h-128 w-full'
					src={getDataUri(contract)}
					ref={ndaPdf}
					data-cy='contract-iframe'
				/>
				<div className='flex'>
					<div className='flex rounded-sm border border-gray-400'>
						<label
							htmlFor='fileUpload'
							className='flex grow items-center justify-center border-custom-blue bg-custom-blue px-3 py-1 text-center text-xl font-semibold text-white hover:bg-custom-light-blue'
							data-cy='signature-file-input-button'>
							{hasUploaded
								? 'Signatur lastet opp'
								: 'Last opp signatur'}
						</label>
						<input
							type='file'
							id='fileUpload'
							onChange={(e) => uploadSignature(e)}
							className='hidden'
							data-cy='signature-file-input'
						/>
						<div
							onClick={() => setIsOpen(true)}
							className='px-3 py-1 text-center text-sm font-semibold text-white hover:bg-gray-700'>
							{'Eller tegn'}
							<div className='flex flex-row items-center justify-center'
								 data-cy='signature-draw-button'>
								<IconSignature size={26} />
								<IconPencil size={26} />
							</div>
						</div>
					</div>
					<button
						onClick={onSend}
						disabled={!hasUploaded}
						style={{
							backgroundColor: hasUploaded ? '#39c16c' : ''
						}}
						className='ml-5 rounded-sm bg-gray-500 px-3 py-1 text-center text-xl font-semibold text-white hover:brightness-125'
						data-cy='send-contract-button'>
						Send
					</button>
				</div>
				<Sign
					isOpen={isOpen}
					setIsOpen={setIsOpen}
					onSave={signatureCanvasOnSave}
				/>
			</div>
		</div>
	);
};

export default SenderSignaturePage;
